/*
 * Copyright (C) 2011 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.amnesica.kryptey.inputmethod.latin.settings;

import android.content.Context;
import android.os.Bundle;
import android.preference.PreferenceFragment;

import com.amnesica.kryptey.inputmethod.compat.PreferenceManagerCompat;

/**
 * This is a helper class for an IME's settings preference fragment. It's recommended for every
 * IME to have its own settings preference fragment which inherits this class.
 */
public abstract class InputMethodSettingsFragment extends PreferenceFragment {
  private final InputMethodSettingsImpl mSettings = new InputMethodSettingsImpl();

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    final Context context = getActivity();
    setPreferenceScreen(getPreferenceManager().createPreferenceScreen(
        PreferenceManagerCompat.getDeviceContext(context)));
    mSettings.init(context, getPreferenceScreen());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void onResume() {
    super.onResume();
    mSettings.updateEnabledSubtypeList();
  }
}
